/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#include "pgmrenderbasewindow.h"

#ifndef __PGM_RENDER_DRAWABLE_H__
#define __PGM_RENDER_DRAWABLE_H__

/* pgmrenderdrawable.h and pgmrenderbasewindow.h include each other */
typedef struct _PgmRenderDrawable PgmRenderDrawable;
typedef struct _PgmRenderDrawableClass PgmRenderDrawableClass;

#include <glib-object.h>
#include <gst/gst.h>
#include "pgmrendererror.h"

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_DRAWABLE            (pgm_render_drawable_get_type ())
#define PGM_RENDER_DRAWABLE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_DRAWABLE, PgmRenderDrawable))
#define PGM_RENDER_DRAWABLE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), PGM_RENDER_TYPE_DRAWABLE, PgmRenderDrawableClass))
#define PGM_RENDER_IS_DRAWABLE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_DRAWABLE))
#define PGM_RENDER_IS_DRAWABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), PGM_RENDER_TYPE_DRAWABLE))
#define PGM_RENDER_DRAWABLE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_RENDER_TYPE_DRAWABLE, PgmRenderDrawableClass))

#define PGM_RENDER_DRAWABLE_CAST(obj)	       ((PgmRenderDrawable *) (obj))
#define PGM_RENDER_DRAWABLE_GET_LOCK(drawable) (PGM_RENDER_DRAWABLE_CAST (drawable)->lock)

/**
 * PGM_RENDER_DRAWABLE_LOCK:
 * @drawable: A #PgmRenderDrawable object.
 *
 * Lock the #drawable instance.
 * FIXME: Need more details.
 */
#define PGM_RENDER_DRAWABLE_LOCK(drawable) g_static_mutex_lock (PGM_RENDER_DRAWABLE_GET_LOCK (drawable))

/**
 * PGM_RENDER_DRAWABLE_TRYLOCK:
 * @drawable: A #PgmRenderDrawable object.
 *
 * Try to lock the #drawable instance if it is not locked yet.
 * FIXME: Need more details.
 */
#define PGM_RENDER_DRAWABLE_TRYLOCK(drawable) g_static_mutex_trylock (PGM_RENDER_DRAWABLE_GET_LOCK (drawable))

/**
 * PGM_RENDER_DRAWABLE_UNLOCK:
 * @drawable: A #PgmRenderDrawable object.
 *
 * Unlock the #drawable instance.
 * FIXME: Need more details.
 */
#define PGM_RENDER_DRAWABLE_UNLOCK(drawable) g_static_mutex_unlock (PGM_RENDER_DRAWABLE_GET_LOCK (drawable))

/**
 * PgmRenderDrawableOrder:
 * @PGM_RENDER_DRAWABLE_UNBINDED: Not binded to a Window.
 * @PGM_RENDER_DRAWABLE_FAR: Orderable drawables drawn behind all the
 *                           other drawable types.
 * @PGM_RENDER_DRAWABLE_Z_ORDERED: z-ordered drawables drawn over the
 *                                 far drawables and behind the near
 *                                 one.
 * @PGM_RENDER_DRAWABLE_NEAR: Orderable drawables drawn over all the
 *                            other drawable types.
 *
 * The rendering order types.
 */
typedef enum {
  PGM_RENDER_DRAWABLE_UNBINDED,
  PGM_RENDER_DRAWABLE_FAR,
  PGM_RENDER_DRAWABLE_Z_ORDERED,
  PGM_RENDER_DRAWABLE_NEAR
} PgmRenderDrawableOrder;

struct _PgmRenderDrawable {
  GObject                parent;

  /*< public >*/
  GStaticMutex           *lock;
  PgmRenderBaseWindow    *window;
  PgmRenderDrawableOrder order;
  gboolean               show; 

  /*< private >*/
};

struct _PgmRenderDrawableClass {
  GObjectClass parent_class;

  /*< public >*/
  GstCaps      *caps;

  PgmRenderError (*draw) (PgmRenderDrawable *drawable);
  PgmRenderError (*hide) (PgmRenderDrawable *drawable);
  PgmRenderError (*show) (PgmRenderDrawable *drawable);

  /*< private >*/
};

GType              pgm_render_drawable_get_type (void) G_GNUC_CONST;
PgmRenderDrawable *pgm_render_drawable_new      (void);
PgmRenderError     pgm_render_drawable_get_caps (PgmRenderDrawable *drawable, GstCaps **caps);
PgmRenderError     pgm_render_drawable_draw     (PgmRenderDrawable *drawable);
PgmRenderError     pgm_render_drawable_hide     (PgmRenderDrawable *drawable);
PgmRenderError     pgm_render_drawable_show     (PgmRenderDrawable *drawable);

G_END_DECLS

#endif /* __PGM_RENDER_DRAWABLE_H__ */
