/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __PGM_RENDER_EVENTS_H__
#define __PGM_RENDER_EVENTS_H__

#include <glib-object.h>

typedef struct _PgmRenderEventAny PgmRenderEventAny;
typedef struct _PgmRenderEventMotion PgmRenderEventMotion;
typedef struct _PgmRenderEventButton PgmRenderEventButton;
typedef struct _PgmRenderEventScroll PgmRenderEventScroll;
typedef struct _PgmRenderEventKey PgmRenderEventKey;
typedef struct _PgmRenderEventConfigure PgmRenderEventConfigure;
typedef struct _PgmRenderEventDelete PgmRenderEventDelete;
typedef struct _PgmRenderEventSource PgmRenderEventSource;
typedef union _PgmRenderEvent PgmRenderEvent;

typedef void (*PgmRenderEventFunc) (PgmRenderEvent *event, gpointer user_data);

#define PGM_RENDER_TYPE_EVENT      (pgm_render_event_get_type ())

typedef enum {
  PGM_RENDER_NOTHING        = -1,
  PGM_RENDER_MOTION_NOTIFY  = 0,
  PGM_RENDER_BUTTON_PRESS   = 1,
  PGM_RENDER_2BUTTON_PRESS  = 2,
  PGM_RENDER_3BUTTON_PRESS  = 3,
  PGM_RENDER_BUTTON_RELEASE = 4,
  PGM_RENDER_KEY_PRESS	    = 5,
  PGM_RENDER_KEY_RELEASE    = 6,
  PGM_RENDER_CONFIGURE      = 7,
  PGM_RENDER_SCROLL         = 8,
  PGM_RENDER_DELETE         = 9
} PgmRenderEventType;

typedef enum {
  PGM_RENDER_BUTTON_LEFT,
  PGM_RENDER_BUTTON_MIDDLE,
  PGM_RENDER_BUTTON_RIGHT
} PgmRenderButton;

typedef enum {
  PGM_RENDER_SCROLL_UP,
  PGM_RENDER_SCROLL_DOWN
} PgmRenderScrollDirection;

/* FIXME: Partial keymap */
typedef enum {
  PGM_RENDER_KEY_SPACE  = 15,
  PGM_RENDER_KEY_RETURN = 18,
  PGM_RENDER_KEY_ESCAPE = 19,
  PGM_RENDER_KEY_LEFT   = 20,
  PGM_RENDER_KEY_UP     = 21,
  PGM_RENDER_KEY_RIGHT  = 22,
  PGM_RENDER_KEY_DOWN   = 23,
  PGM_RENDER_KEY_a      = 50,
  PGM_RENDER_KEY_b      = 51,
  PGM_RENDER_KEY_c      = 52,
  PGM_RENDER_KEY_d      = 53,
  PGM_RENDER_KEY_e      = 54,
  PGM_RENDER_KEY_f      = 55,
  PGM_RENDER_KEY_g      = 56,
  PGM_RENDER_KEY_h      = 57,
  PGM_RENDER_KEY_i      = 58,
  PGM_RENDER_KEY_j      = 59,
  PGM_RENDER_KEY_k      = 60,
  PGM_RENDER_KEY_l      = 61,
  PGM_RENDER_KEY_m      = 62,
  PGM_RENDER_KEY_n      = 63,
  PGM_RENDER_KEY_o      = 64,
  PGM_RENDER_KEY_p      = 65,
  PGM_RENDER_KEY_q      = 66,
  PGM_RENDER_KEY_r      = 67,
  PGM_RENDER_KEY_s      = 68,
  PGM_RENDER_KEY_t      = 69,
  PGM_RENDER_KEY_u      = 70,
  PGM_RENDER_KEY_v      = 71,
  PGM_RENDER_KEY_w      = 72,
  PGM_RENDER_KEY_x      = 73,
  PGM_RENDER_KEY_y      = 74,
  PGM_RENDER_KEY_z      = 75
} PgmRenderKeyValue;

/* Warning: all members of _PgmRenderEventAny must
 * be replicated in all the _PgmRenderEvent* in the exact
 * same order */
struct _PgmRenderEventAny {
  PgmRenderEventType type;
  guint8             source;
};

struct _PgmRenderEventMotion {
  PgmRenderEventType type;
  guint8             source;
  guint32            time;
  gdouble            x;
  gdouble            y;
};

struct _PgmRenderEventButton {
  PgmRenderEventType type;
  guint8             source;
  guint32            time;
  gdouble            x;
  gdouble            y;
  guint              button;
};

struct _PgmRenderEventScroll {
  PgmRenderEventType       type;
  guint8                   source;
  guint32                  time;
  gdouble                  x;
  gdouble                  y;
  PgmRenderScrollDirection direction;
};

struct _PgmRenderEventKey {
  PgmRenderEventType type;
  guint8             source;
  guint32            time;
  guint              keyval;
/*   guint16            hardware_keycode; */
};

struct _PgmRenderEventConfigure {
  PgmRenderEventType type;
  guint8             source;
  guint32            time;
  gint               x;
  gint               y;
  gint               width;
  gint               height;
};

struct _PgmRenderEventDelete {
  PgmRenderEventType type;
  guint8             source;
  guint32            time;
};

union _PgmRenderEvent {
  PgmRenderEventType      type;
  PgmRenderEventAny       any;
  PgmRenderEventMotion    motion;
  PgmRenderEventButton    button;
  PgmRenderEventScroll    scroll;
  PgmRenderEventKey       key;
  PgmRenderEventConfigure configure;
  PgmRenderEventDelete    delete;
};

GType           pgm_render_event_get_type (void) G_GNUC_CONST;
PgmRenderEvent *pgm_render_event_new      (PgmRenderEventType type);
PgmRenderEvent *pgm_render_event_copy     (PgmRenderEvent *event);
void            pgm_render_event_free     (PgmRenderEvent *event);

#endif /* __PGM_RENDER_EVENTS_H__ */
