/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment Render base library
 *
 * Copyright © 2006 Fluendo Embedded S.L. <www.fluendo.com>
 * All rights reserved.
 *
 * This file is available under one of two license agreements.
 *
 * This file is licensed under the GPL version 2. See "LICENSE.GPL" in
 * the root of this distribution including a special exception to use
 * Pigment with Fluendo's plugins.
 *
 * The GPL part of Pigment is also available under a commercial licensing
 * agreement from Fluendo. See "LICENSE.Pigment" in the root directory
 * of this distribution package for details on that license.
 */

#ifndef __PGM_RENDER_SHADOW_IFACE_H__
#define __PGM_RENDER_SHADOW_IFACE_H__

#include "pgmrendererror.h"
#include <glib-object.h>

G_BEGIN_DECLS

#define PGM_RENDER_TYPE_SHADOW_IFACE             (pgm_render_shadow_iface_get_type ())
#define PGM_RENDER_SHADOW_IFACE(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_RENDER_TYPE_SHADOW_IFACE, PgmRenderShadowIFace))
#define PGM_RENDER_SHADOW_IFACE_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST((vtable), PGM_RENDER_TYPE_SHADOW_IFACE, GTypeInterface))
#define PGM_RENDER_IS_SHADOW_IFACE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_RENDER_TYPE_SHADOW_IFACE))
#define PGM_RENDER_IS_SHADOW_IFACE_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE((vtable), PGM_RENDER_TYPE_SHADOW_IFACE))
#define PGM_RENDER_SHADOW_IFACE_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_INTERFACE((inst), PGM_RENDER_TYPE_SHADOW_IFACE, PgmRenderShadowIFaceClass))

/**
 * PgmRenderShadowType:
 * @PGM_RENDER_HARD_SHADOW: Shadow with a unique color.
 * @PGM_RENDER_SOFT_SHADOW: Shadow with a blurred color.
 *
 * The different types of shadow.
 */
typedef enum {
  PGM_RENDER_HARD_SHADOW,
  PGM_RENDER_SOFT_SHADOW
} PgmRenderShadowType;

typedef struct _PgmRenderShadowIFace      PgmRenderShadowIFace;
typedef struct _PgmRenderShadowIFaceClass PgmRenderShadowIFaceClass;

struct _PgmRenderShadowIFaceClass {
  GTypeInterface parent;

  PgmRenderError (*set_radius) (PgmRenderShadowIFace *shadow, gint radius);
  PgmRenderError (*get_radius) (PgmRenderShadowIFace *shadow, gint *radius);
  PgmRenderError (*set_offset) (PgmRenderShadowIFace *shadow, gint x, gint y);
  PgmRenderError (*get_offset) (PgmRenderShadowIFace *shadow, gint *x, gint *y);
  PgmRenderError (*set_type)   (PgmRenderShadowIFace *shadow, PgmRenderShadowType type);
  PgmRenderError (*get_type)   (PgmRenderShadowIFace *shadow, PgmRenderShadowType *type);
  PgmRenderError (*set_color)  (PgmRenderShadowIFace *shadow, guchar r, guchar g, guchar b, guchar a);
  PgmRenderError (*get_color)  (PgmRenderShadowIFace *shadow, guchar *r, guchar *g, guchar *b, guchar *a);
};

GType          pgm_render_shadow_iface_get_type (void) G_GNUC_CONST;
PgmRenderError pgm_render_shadow_set_radius     (PgmRenderShadowIFace *shadow, gint radius);
PgmRenderError pgm_render_shadow_get_radius     (PgmRenderShadowIFace *shadow, gint *radius);
PgmRenderError pgm_render_shadow_set_offset     (PgmRenderShadowIFace *shadow, gint x, gint y);
PgmRenderError pgm_render_shadow_get_offset     (PgmRenderShadowIFace *shadow, gint *x, gint *y);
PgmRenderError pgm_render_shadow_set_type       (PgmRenderShadowIFace *shadow, PgmRenderShadowType type);
PgmRenderError pgm_render_shadow_get_type       (PgmRenderShadowIFace *shadow, PgmRenderShadowType *type);
PgmRenderError pgm_render_shadow_set_color      (PgmRenderShadowIFace *shadow, guchar r, guchar g, guchar b, guchar a);
PgmRenderError pgm_render_shadow_get_color      (PgmRenderShadowIFace *shadow, guchar *r, guchar *g, guchar *b, guchar *a);

G_END_DECLS

#endif /* __PGM_RENDER_SHADOW_IFACE_H__ */
